/*=========================================

	VArray -- utility code for working with
	Handle-based arrays with variable number
	of elements.

	 2000-2001 Brian 'Moses' Hall
	moses@blugs.com
	www.blugs.com

	Heapsort implementation based on code by
	Michael Schrig and Kyle Hammond.

=========================================*/


#ifndef __VARRAY__
#define __VARRAY__

#ifndef __MACMEMORY__
#include <MacMemory.h>
#endif

#ifndef __MACERRORS__
#include <MacErrors.h>
#endif

#ifndef __VA_INTERNAL__
typedef struct OpaqueVAReference*			VARef;
#endif


#ifndef VA_PROC
	#if defined( __cplusplus )
		#define VA_PROC	extern "C"
	#else
		#define VA_PROC
	#endif
#endif

#define va_OffsetOf(type,field)			((UInt32)(&(((type*)0)->field)))

/* Returned by comparison callbacks. */
typedef SInt32	VAComparisonResult;
enum
{
	vaSearchDataIsGreater		= -1L,
	vaEqual						= 0,
	vaSearchDataIsLess			= 1,
	vaNotEqual					= 1		/* This one is for brute-force only. */
};

enum
{
	vaFirstIsLess		= -1L,
	vaFirstIsGreater	= 1
};

/* Returned by iteration callback. */
typedef UInt32	VAIteratorResult;
enum
{
	vaIteratorKeepGoing,
	vaIteratorStop
};


enum
{
	vaNoMovementNecessaryErr	= 2600	// Not really an error, VAValidateMoveParameters
										//	and VAMoveAfter return it when the parameters
										//	are technically OK but would result in no movement.
										//	Yes, I numbered it after everybody's favorite
										//	hacker mag.
};

/* Callbacks. */
typedef VAComparisonResult	(*VASearchProcPtr)(	const Ptr inArrayDataP,
												void* inSearchData,
												UInt32 inRefCon );

typedef VAComparisonResult	(*VABruteForceSearchProcPtr)
											(	const Ptr inArrayDataP,
												void* inSearchData,
												UInt32 inRefCon );

typedef VAComparisonResult	(*VASortProcPtr)(	const Ptr inData1P,
												const Ptr inData2P,
												UInt32 inRefCon );

// Pass data between app and iterator using ioUserData.
typedef VAIteratorResult	(*VAIteratorPtr)(	Ptr inArrayDataP,
												void* ioUserData );




VA_PROC VARef	VANew( UInt32 inItemCount, UInt32 inItemSize, Boolean inZero, UInt32 inRefCon );
VA_PROC void	VADispose( VARef inArray );

VA_PROC UInt32	VACount( VARef inArray );
VA_PROC UInt32	VAItemSize( VARef inArray );
VA_PROC UInt32	VARefCon( VARef inArray );
VA_PROC void	VASetRefCon( UInt32 inRefCon, VARef inArray );
VA_PROC UInt32	VAOwnerCount( VARef inArray );

VA_PROC void*	VAAdd( UInt32 inCount, UInt32 inAddHere, Boolean inZero, VARef inArray );
VA_PROC void*	VAAddToEnd( UInt32 inCount, Boolean inZero, VARef inArray );
VA_PROC OSErr	VAInsert( UInt32 inAddHere, void* inData, VARef inArray );
VA_PROC OSErr	VAInsertAtEnd( void* inData, VARef inArray );
VA_PROC void	VARemove( UInt32 inCount, UInt32 inItem, VARef inArray );
VA_PROC void	VARemoveAll( VARef inArray );
VA_PROC OSErr	VAMove( UInt32 inCount, UInt32 inSource, UInt32 inDest, VARef inArray );
VA_PROC OSErr	VAMoveAfter( UInt32 inCount, UInt32 inSource, UInt32 inDest, VARef inArray );
VA_PROC OSErr	VAValidateMoveParameters( UInt32 inCount, UInt32 inSource, UInt32 inDest,
											UInt32 inNArrayItems );
VA_PROC OSErr	VASwap( UInt32 inItem1, UInt32 inItem2, VARef inArray );
VA_PROC UInt32	VASearch( VASearchProcPtr inProc, UInt32 inRefCon, void* inSearchData,
							UInt32* outNext, VARef inArray );
VA_PROC UInt32	VABruteForceSearch( VABruteForceSearchProcPtr inProc, UInt32 inRefCon,
									void* inSearchData, VARef inArray );
VA_PROC void	VASort( VASortProcPtr inProc, UInt32 inRefCon, VARef inArray );
VA_PROC void	VAZeroMemory( Ptr inPtr, UInt32 inBytes );
VA_PROC void	VASwapMemory( Ptr inPtr1, Ptr inPtr2, UInt32 inBytes );
VA_PROC void*	VANthItemPtr( UInt32 inNthItem, VARef inArray );
VA_PROC void*	VALastItemPtr( VARef inArray );
VA_PROC OSErr	VAGetNthItem( UInt32 inNthItem, void* outData, VARef inArray );
VA_PROC OSErr	VASetNthItem( UInt32 inItem, void* inData, VARef inArray );
VA_PROC VARef	VACopy( VARef inSourceArray, VARef inTargetArray );
VA_PROC VARef	VACopyPart( UInt32 inCount, VARef inArray );
VA_PROC VARef	VAClone( VARef inArray );
VA_PROC OSErr	VAConcatenate( VARef inSource, VARef inDest );
VA_PROC void	VAIterate( VAIteratorPtr inProc, void* ioUserData, VARef inArray );


VA_PROC void	VAInternalSort( Size inSize, Boolean inSigned, UInt32 inOffset, VARef inArray );
VA_PROC UInt32	VAInternalSearch( Size inSize, Boolean inSigned, UInt32 inOffset, void* inSearchData,
							UInt32* outNext, VARef inArray );
VA_PROC UInt32	VAInternalBruteForceSearch( Size inSize, Boolean inSigned, UInt32 inOffset,
									void* inSearchData, VARef inArray );
VA_PROC void	VANewNumberOnMovement( UInt32* ioNumber, UInt32 inCount, UInt32 inSource, UInt32 inDest );
VA_PROC void	VANewNumberOnAddition( UInt32* ioNumber, UInt32 inCount, UInt32 inAddHere );
VA_PROC void	VANewNumberOnDeletion( UInt32* ioNumber, UInt32 inCount, UInt32 inDeleteHere );




#endif //__VARRAY__
