	MACRO
	MakeFunction &fnName
		EXPORT &fnName[DS]
 		EXPORT .&fnName[PR]
		
		TC &fnName[TC], &fnName[DS]
			
		CSECT &fnName[DS]
			DC.L .&fnName[PR]
 			DC.L TOC[tc0]
		
		CSECT .&fnName[PR]
		FUNCTION .&fnName[PR]	
		
	ENDM
	
	
	toc
		tc gZeroDouble[TC], gZeroDouble[ro]
		
	
	csect gZeroDouble[ro]
		dc.l 0x00000000
		dc.l 0x00000000

	
	MakeFunction	VAZeroMemory
;===============================
; p is in r3, bytes in r4, zero in r0.

	cmplwi		r3,0			; Check for nil pointer
	beqlr-						; Return if equal (to nil)
	li			r0,0			; Load zero value
	cmplwi		r4,16			; If less than 16 bytes to copy...
	ble-		@SetupCopyByte	;	copy byte by bye at end
@AlignByte
	rlwinm.		r6,r3,0,31,31	; Check bit 31 (ones place)
	beq+		@AlignShort		; If bit 31 is clear, try to align to short
	stb			r0,0(r3)		; Store zero byte to ptr
	addi		r3,r3,1			; Increment ptr
	subi		r4,r4,1			; Decrement bytes to copy
@AlignShort
	rlwinm.		r6,r3,0,30,30	; Check bit 30 (twos place)
	beq+		@AlignLong		; If bit 30 is clear, try to align to long
	sth			r0,0(r3)		; Store zero short to ptr
	addi		r3,r3,2			; Increment ptr
	subi		r4,r4,2			; Decrement bytes to copy
@AlignLong
	rlwinm.		r6,r3,0,29,29	; Check bit 29 (fours place)
	beq			@SetupCopyDouble; If bit 29 is clear, try to align to double
	stw			r0,0(r3)		; Store zero long to ptr
	addi		r3,r3,4			; Increment ptr
	subi		r4,r4,4			; Decrement bytes to copy
@SetupCopyDouble
	lwz			r6,gZeroDouble[TC](RTOC)
	lfd			fp0,0(r6)		; Set fp register 0 to zero value. There is no
								;	floating-point load immediate instruction (too bad) so we
								;	have to use a global.
	subi		r3,r3,8			; Back up so store double with update instruction (stfdu)
								;	can add 8 to ptr to get store location
@CheckDouble
	cmplwi		r4,8			; At least 8 bytes left to copy?
	blt-		@DoneWithDouble	; Clean up if not
@ZeroDouble
	stfdu		fp0,8(r3)		; Store double to memory offset by immediate value
								;	(in this case, 8) and increment r3 by that amount
	subi		r4,r4,8			; Decrement bytes to copy
	b			@CheckDouble	; Loop over double-storing code
@DoneWithDouble
	addi		r3,r3,8			; Reset r3 to point to next byte to be zeroed
@SetupCopyByte
	subi		r3,r3,1			; Do the same trick as with doubles, except we
								;	only 'autoincrement' by 1
@CheckByte
	cmplwi		r4,0			; Zero bytes left to copy?
	beqlr-						; Return if so
@CopyByte
	stbu		r0,1(r3)		; Store byte to memory offset by the immediate value (1)
								;	and autoincrement by 1. This is like saying *(++p) = 0;
	subi		r4,r4,1			; Decrement bytes left
	b			@CheckByte		; Loop over byte-storing (exit condition is there)


	csect .VAZeroMemory[pr]
	dc.b 'VAZeroMemory'

	
	MakeFunction VASwapMemory
;===============================
; r0:			rlwinm result, storage
; r3:			inPtr1
; r4:			inPtr2
; r5:			inBytes
; r6:			storage
; fp1, fp2:	storage

	cmplw		r3,r4			; See if blocks are the same
	beqlr-						; Return if equal
	cmplwi		r4,16			; If less than 16 bytes to copy...
	blt-		@Bytes			;	copy byte by bye at end
	rlwinm.		r0,r3,0,31,31	; Check bit 31 (ones place)
	beq			@CheckPtr2HalfWordAligned
								; If bit 31 is set, try to align to short
	lbz			r0,0(r3)		; Get a byte from inPtr1
	lbz			r6,0(r4)		; Get a byte from inPtr2
	stb			r0,0(r4)		; Store byte in inPtr2
	stb			r6,0(r3)		; Store byte in inPtr1
	addi		r3,r3,1			; Increment pointers...
	addi		r4,r4,1
	subi		r5,r5,1			; Decrement bytes to copy
@CheckPtr2HalfWordAligned
	rlwinm.		r0,r4,0,31,31	; Check bit 31 (ones place) for inPtr2.
	bne-		@Bytes			; If odd, we can't halfword-align the two pointers
	rlwinm.		r0,r3,0,30,30	; Check bit 30 (twos place)
	beq			@CheckPtr2WordAligned
								; If bit 30 is set, try to align to word
	lhz			r0,0(r3)		; Get a half word from inPtr1
	lhz			r6,0(r4)		; Get a half word from inPtr2
	sth			r0,0(r4)		; Store half word in inPtr2
	sth			r6,0(r3)		; Store half word in inPtr1
	addi		r3,r3,2			; Increment pointers...
	addi		r4,r4,2
	subi		r5,r5,2			; Decrement bytes to copy
@CheckPtr2WordAligned
	rlwinm.		r0,r4,0,30,30	; Check bit 30 (twos place) for inPtr2.
	bne			@HalfWords		; If not multiple of 4, we can't word-align the two pointers
	rlwinm.		r0,r3,0,29,29	; Check bit 29 (fours place)
	beq			@CheckPtr2DoubleAligned
								; If bit 29 is set, try to align to double
	lwz			r0,0(r3)		; Get a word from inPtr1
	lwz			r6,0(r4)		; Get a word from inPtr2
	stw			r0,0(r4)		; Store word in inPtr2
	stw			r6,0(r3)		; Store word in inPtr1
	addi		r3,r3,4			; Increment pointers...
	addi		r4,r4,4
	subi		r5,r5,4			; Decrement bytes to copy
@CheckPtr2DoubleAligned
	rlwinm.		r0,r4,0,29,29	; Check bit 29 (fours place) for inPtr2.
	bne			@Words			; If not multiple of 8, we can't double-align the two pointers
@Doubles
	cmplwi		r5,8			; Compare remaining bytes to 8
	blt-		@Words			; If less than 8 bytes, we can't copy a double
	lfd			fp0,0(r3)		; Get a double from inPtr1
	lfd			fp1,0(r4)		; Get a double from inPtr2
	stfd		fp0,0(r4)		; Store double in inPtr2
	stfd		fp1,0(r3)		; Store double in inPtr1
	addi		r3,r3,8			; Increment pointers...
	addi		r4,r4,8
	subi		r5,r5,8			; Decrement bytes to copy
	b			@Doubles		; Go back to check bytes left
@Words
	cmplwi		r5,4			; Compare remaining bytes to 4
	blt-		@HalfWords		; If less than 4 bytes, we can't copy a word
	lwz			r0,0(r3)		; Get a word from inPtr1
	lwz			r6,0(r4)		; Get a word from inPtr2
	stw			r0,0(r4)		; Store word in inPtr2
	stw			r6,0(r3)		; Store word in inPtr1
	addi		r3,r3,4			; Increment pointers...
	addi		r4,r4,4
	subi		r5,r5,4			; Decrement bytes to copy
	b			@Words			; Go back to check bytes left
@HalfWords
	cmplwi		r5,2			; Compare remaining bytes to 2
	blt-		@Bytes			; If less than 2 bytes, we can't copy a half word
	lhz			r0,0(r3)		; Get a half word from inPtr1
	lhz			r6,0(r4)		; Get a half word from inPtr2
	sth			r0,0(r4)		; Store half word in inPtr2
	sth			r6,0(r3)		; Store half word in inPtr1
	addi		r3,r3,2			; Increment pointers...
	addi		r4,r4,2
	subi		r5,r5,2			; Decrement bytes to copy
	b			@HalfWords		; Go back to check bytes left
@Bytes
	subi		r3,r3,1			; Set up for the load/store with update instructions
	subi		r4,r4,1			;	we only do this once
@BytesNowSetup
	cmplwi		r5,1			; Compare remaining bytes to 1
	bltlr-						; EXIT if no bytes left
	lbzu		r0,0(r3)		; Increment inPtr1 and get a byte
	lbzu		r6,0(r4)		; Increment inPtr2 and get a byte
	stb			r0,0(r4)		; Store byte in inPtr2
	stb			r6,0(r3)		; Store byte in inPtr1
	subi		r5,r5,1			; Decrement bytes to copy
	b			@BytesNowSetup	; Go back to check bytes left
	
	csect .VASwapMemory[pr]
	dc.b 'VASwapMemory'
